package ru.telamon.topcoderparser;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.psi.PsiDirectory;
import com.intellij.psi.PsiElementFactory;
import com.intellij.psi.PsiFile;

import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.io.BufferedReader;
import java.io.StringReader;

public class TopCoderParserAction extends AnAction {

    public static void main(String[] args) {
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();

        DataFlavor[] dataFlavors = clipboard.getAvailableDataFlavors();

        for (DataFlavor dataFlavor : dataFlavors) {
            if (dataFlavor == DataFlavor.stringFlavor) {
                try {
                    System.out.println(getCode((String) clipboard.getData(dataFlavor)));
                }
                catch (Exception ex) {
                    ex.printStackTrace();
                }

                break;
            }
        }
    }

    public static String getValue(BufferedReader in, String prefix) throws Exception {
        while (true) {
            String v = in.readLine();
            if (v == null)
                return null;
            v = v.trim();
            if (v.startsWith(prefix))
                return in.readLine().trim();
        }
    }

    public static String getVariable(BufferedReader in, String name) throws Exception {
        boolean isArray = name.contains("[]");
        String longSuffix = name.contains("long") ? "L" : "";
        String s;
        do {
            s = in.readLine();
            s = s.trim();
        } while (s.length() == 0) ;

        String returnStr = "Returns:";

        if (s.startsWith(returnStr))
            s = s.substring(returnStr.length());

        if (!isArray)
            return s + longSuffix;

        String ans = "new " + name;
        while (!s.endsWith("}")) {
            ans += s + longSuffix;
            s = in.readLine();
        }
        ans += s;
        return ans;
    }

    public static String[] getCode(String problem) throws Exception {        
        problem = problem.replace(((char) 160), ' ');
        String ans = "";
        ans += "import java.util.*;\n\n";
        BufferedReader in = new BufferedReader(new StringReader(problem));
        String className = getValue(in, "Class:");
        String methodName = getValue(in, "Method:");
        String[] parameters = getValue(in, "Parameters:").split(", ");
        String returnType = getValue(in, "Returns:");
        String methodSignature = getValue(in, "Method signature:");
        ans += "public class " + className + " {\n";


        ans += "    public " + methodSignature + "{\n";
        ans += "    }\n\n\n\n\n\n\n";
        ans += "    public static void main(String[] args) {\n" +
               "        RETester.test(" + className +  ".class, \"test.*\");\n" +
               "    }\n";


        int test = 0;
        while (getValue(in, test + ")") != null) {
            ans += "    public void test" + test + "(){\n";
            ans += "        RETester.eq(" + methodName + "(";
            boolean started = false;
            for (String par : parameters) {
                if (started)
                    ans += ", ";
                ans += getVariable(in, par);
                started = true;
            }
            ans += "),\n            ";
            ans += getVariable(in, returnType);
            ans += ");\n";
            ans += "    }\n\n";
            test++;
        }

        ans += "}";
        return new String[]{className, ans};
    }

    public void actionPerformed(AnActionEvent e) {
        Application application =
                ApplicationManager.getApplication();

        final DataContext dataContext = e.getDataContext();
        application.runWriteAction(new Runnable() {
            public void run() {

                final PsiDirectory element = (PsiDirectory) dataContext.getData(DataConstants.PSI_ELEMENT);
                PsiElementFactory factory = element.getManager().getElementFactory();
                Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();

                DataFlavor[] dataFlavors = clipboard.getAvailableDataFlavors();

                for (DataFlavor dataFlavor : dataFlavors) {
                    if (dataFlavor == DataFlavor.stringFlavor) {
                        try {
                            String[] code = getCode((String) clipboard.getData(dataFlavor));
                            PsiFile file = factory.createFileFromText(code[0] + ".java", code[1]);
                            element.add(file);

                        }
                        catch (Exception ex) {
                            ex.printStackTrace();
                        }

                        break;
                    }
                }
            }
        });
    }
}
